;  Teeny little x86 PC bootsector program 
;  Designed for the Netwide assembler, "nasm".
;  Compile into a boot block with "nasm -f bin boot.s -o boot.bin"
;   See http://nasm.sourceforge.net/doc/html/nasmdoc0.html
;  Orion Sky Lawlor, olawlor@acm.org, 2005/12/20 (Public Domain)

BITS 16         ; Starts running in 16-bit segmented DOS mode
SECTION .text   ; Put everything into the code ("text") segment
GLOBAL _start   ; Tells linker where code starts

_start: ; BIOS loads this code, and starts running right here

%define rows 24
%define cols 80
%define evil 0x0E  ; DOS character to use for bad guys 
	mov ebp,esp; Set up base pointer
	sub esp,32; Make room for local variables
	sub esp,(rows+1)*cols; Make room for big array of chars
	mov edi,esp; Start of x array
	sub esp,cols ; spare row
	sub esp,cols; Make room for row-sized output buffer
	
%define userRow long [ebp-4]
%define userCol long [ebp-8]
%define randSeed long [ebp-12]
	mov userRow,rows-1 ; User's row
	mov userCol,cols/2 ; User's column
	mov randSeed,123 ; Random number seed

; Clear storage array at edi
	mov eax,0
clear:
	mov byte [edi+eax],' '
	inc eax
	cmp eax,rows*cols
	jl clear
	
; Add brick of evil dudes
	mov byte [edi+20],evil
	mov byte [edi+21],evil
	mov byte [edi+22],evil
	
redraw:
	; Move cursor to screen top-right
	mov ah,0x02
	mov bh,0x00
	mov dx,0
	int 0x10
	
; Loop over screen:
	mov ebx,edi ; Points to start of ordinance/evil row
	mov edx,0 ; Line counter (increases)
drawLine:
	
;   Fill line with spaces and objects
	mov ecx,0 ; Column counter (increases)
spaces:
	mov al, byte[ebx+ecx]
	mov byte [esp+ecx],al
	inc cx
	cmp cx,cols
	jl spaces
	
;   Draw in everything onscreen
; Advance ordinance
	mov ecx,0 ; Column counter (increases)
ordnance_loop:
	cmp byte [ebx+ecx],'|'
	jne next_ord
	; We're sitting on ordinance--advance it
	mov byte [ebx+ecx],' ' ; erase old position
	mov eax,ebx
	add eax,ecx
	sub eax,cols ; Jump to higher row
	mov byte [eax],'|' ; Draw new position
next_ord:
	inc cx
	cmp cx,cols
	jl ordnance_loop

; Draw user
	cmp edx,userRow
	jne skip_user
	mov ecx,userCol ; user column
	mov byte [esp+ecx],1 ; Draw in user in his column
	mov byte [ebx+ecx],'|' ; Draw in user's ordinance
skip_user:
	
;   Copy line to screen
	mov ah,0x0e
	mov cx,0 ; Column counter (increases)
onscreen:
	mov al,[esp+ecx]
	int 0x10
	inc cx  
	cmp cx,cols
	jl onscreen

; Y loop ++
	add ebx,cols
	inc dx   
	cmp dx,rows
	jl drawLine
; } Y loop


; Advance evil
	mov edx,rows-1 ; Line counter (decreases)
	mov ebx,edi ; Points to start of ordinance/evil row
	add ebx,(rows-1)*cols
evil_line:
	mov ecx,0 ; Column counter (increases)
evil_loop:
	cmp byte [ebx+ecx],evil
	jne next_evil
	; We're sitting on evil--advance it
	cmp dx,0
	je skip_erase ; On row 0, the worm does not die
	mov byte [ebx+ecx],' ' ; erase old position
skip_erase:
	mov eax,ebx
	add eax,cols ; Jump to lower row
	cmp byte [eax+ecx],'|'; Hit ordinance!
	je dead_evil
	mov byte [eax+ecx],evil ; Draw new position
	jmp next_evil
dead_evil:
	; Just continue (boring)
next_evil:
	inc cx
	cmp cx,cols
	jl evil_loop
	
; Y loop --
	sub ebx,cols
	dec dx
	jge evil_line
; } Y loop



; Update random numbers
	mov eax,randSeed
	add eax,123
	imul eax,16807
	mov randSeed,eax
	
; Decide if we should generate new evil this time around
	sar eax,8
	and eax,0x1fff  ; Controls difficulty-- evil generation rate
	cmp eax,cols-2 ; only generate evil if eax is small
	jl generateEvil
	jmp noEvil
	
generateEvil:
	mov byte [edi+eax],evil
	inc eax
	mov byte [edi+eax],evil
	inc eax
	mov byte [edi+eax],evil

noEvil:

; Slow down the action to a playable rate
	mov eax,0x100000  ; STOOPID busy-wait loop.  FIXME: figure out how to pause
stoopid: dec eax
	jne stoopid
	
	
; Check for a keypress
	mov ah,0x01
	int 0x16
	jz skip_keystroke
	; Got a keystroke
	mov ah,0x00 ; Grab the keystroke
	int 0x16
	mov bx,0
	cmp al,'a'
	je moveLeft
	cmp al,'d'
	je moveRight
	jmp skip_keystroke ; Uninteresting key
	
moveLeft:
	dec userCol
	jmp checkPos
moveRight:
	inc userCol
	jmp checkPos

checkPos:
	mov eax,userCol
	jl badPos
	cmp eax,80
	jge badPos
	jmp skip_keystroke
badPos:
	mov userCol,40
	
skip_keystroke:
	
	jmp redraw

reboot:
	jmp  0xf000:0xfff0 ; Jump to BIOS reset code

; The boot sector's last two bytes must contain this magic flag.
; This tells the BIOS it's OK to boot here; without this, 
;  you'll get the BIOS "Operating system not found" error.
	times 510-($-_start) db 0  ; pad to offset 510 
	dw 0xAA55 ; Magic value needed by BIOS
END
