;  Teeny little x86 PC bootsector program 
;  Designed for the Netwide assembler, "nasm".
;  Compile into a boot block with "nasm -f bin boot.s -o boot.bin"
;   See http://nasm.sourceforge.net/doc/html/nasmdoc0.html
;  Orion Sky Lawlor, olawlor@acm.org, 2005/12/20 (Public Domain)

BITS 16         ; Starts running in 16-bit segmented DOS mode
SECTION .text   ; Put everything into the code ("text") segment
GLOBAL _start   ; Tells linker where code starts

_start: ; BIOS loads this code, and starts running right here

; Change "video mode" to VGA mode: 320x200 pixels
	mov ax, 0x0013
	int 0x10  ; choose video mode 0x13

; -------------------- Output -------------------------
; Load up address registers to point to VGA display area
; See http://www.faqs.org/faqs/assembly-language/x86/general/part1/section-12.html
	mov ax, 0xa000 ; Segment of VGA screen buffer
	mov es, ax ; "Extra Segment" register
	mov al, 0
	
	sub sp, 32 ; space used during run

drawScreen:
	; begin Y loop:
	mov di, 0 ; Onscreen pointer (counts up)
	mov edx, 0 ; Y counter (counts up)
drawrow:
	; begin x loop
	mov ecx, 0 ; X counter (counts up)
drawpixel:
; ------------------------ Pixel color selection ----------------
;  This is the Mandelbrot set computed with fixed-point arithmetic.
;  So instead of using a float f, I use the int (f*(1<<12)), 
;  which is called .12 fixed-point.
	
	; choose screen color
	mov [esp+0], ecx ; save counters
	mov [esp+4], edx 
	sub ecx,160   ; put origin in center of screen
	sub edx,100
	imul ecx,20  ; ==.12 fixed-point scale / geometric scaling
	imul edx,20
	sub edx,14*256; Zoom in on interesting place
	
	mov [esp+ 8], ecx ; c.real (saved since we're out of registers!)
	mov [esp+12], edx ; c.imag
	
	mov eax,ecx ; z.real
	mov ebx,edx ; z.imag
	mov si,0    ; mandelbrot iteration counter

mloop:
	; z = z*z + c
	;  z.real = z.real*z.real - z.imag*z.imag + c.real
	;  z.imag = 2*z.real*z.imag + c.imag
	; mag_sqr = z.real*z.real+z.imag*z.imag
	
	; Compute squares
	mov ecx,eax
	imul ecx,ecx
	sar ecx,12 ; ecx==z.real*z.real

	mov edx,ebx
	imul edx,edx
	sar edx,12 ; edx==z.imag*z.imag
	
	; Check magnitude
	mov ebp,ecx
	add ebp,edx ; ebp = mag_sqr
	cmp ebp,4*16*256; ==4.0 in .12 fixed-point
	jg mend ; z's magnitude is too big--jump to end.
	
	; Compute new z.real = z.real*z.real - z.imag*z.imag + c.real
	mov ebp,ecx
	sub ebp,edx
	add ebp,[esp+8] ; +=c.real
	
	; Compute new z.imag
	imul ebx,eax     ; ebx = z.real*z.imag
	sar ebx,12 ; .12 fixed-point multiply
	imul ebx,2       ; *=2
	add ebx,[esp+12] ; +=c.imag

	mov eax,ebp
	
	; Increment iteration counter
	inc si
	cmp si,256
	jl mloop ; stops when si gets too big	
mend:
	;mov eax,ebp ; Output color==magnitude
	;sar eax,5 ; Slow down magnitude change rate
	mov ax,si  ; Output color==counter at magnitude trip
	
	mov ecx,[esp+0] ; restore loop counters
	mov edx,[esp+4]
	
	mov [es:di],al  ; Write this byte to the screen
	inc di
	inc cx
	cmp cx,320
	jl drawpixel
	; end x loop
	
	inc dx
	cmp dx,200
	jl drawrow
	; End Y loop
	
	jmp drawScreen
	
	add sp,32

reboot:
	jmp  0xf000:0xfff0 ; Jump to BIOS reset code

; The boot sector's last two bytes must contain this magic flag.
; This tells the BIOS it's OK to boot here; without this, 
;  you'll get the BIOS "Operating system not found" error.
	times 510-($-_start) db 0  ; pad to offset 510 
	dw 0xAA55 ; Magic value needed by BIOS
END
