;  Teeny little x86 PC bootsector program 
;  Designed for the Netwide assembler, "nasm".
;  Compile into a boot block with "nasm -f bin boot.s -o boot.bin"
;   See http://nasm.sourceforge.net/doc/html/nasmdoc0.html
;  Orion Sky Lawlor, olawlor@acm.org, 2005/12/20 (Public Domain)

BITS 16         ; Starts running in 16-bit segmented DOS mode
SECTION .text   ; Put everything into the code ("text") segment
GLOBAL _start   ; Tells linker where code starts

_start: ; BIOS loads this code, and starts running right here

; Change "video mode" to VGA mode: 320x200 pixels
	mov ax, 0x0013
	int 0x10  ; choose video mode 0x13

; -------------------- Color Palette -------------------
; see http://www.inversereality.org/tutorials/graphics%20programming/palettesetting.html
	; Write 0xff to the VGA "mask register" 0x3c6
	mov dx, 0x3c6
	mov al, 0xff
	out dx,al
	
	; Write 0x00 to the VGA start color register 0x3c8
	mov dx, 0x3c8
	mov al, 0
	out dx,al
	
	; Write greyscale R, G, B to register 0x3c9
	;  This is funky for two reasons:
	;    - The VGA card only uses the low 6 bits of each color,
	;       so 63 is actually full-white (not 256!)
	;    - The card counts "out" calls--just issue 3*256 of them!
	mov dx, 0x3c9
nextColor:
	times 12 out dx,al  ; 12=3*4=Set 3 RGB colors for 4 table entries
	inc al
	cmp al, 64  ; al is used as a 6-bit brightness counter
	jl nextColor

; -------------------- Output -------------------------
; Load up address registers to point to VGA display area
; See http://www.faqs.org/faqs/assembly-language/x86/general/part1/section-12.html
	mov ax, 0xa000 ; Segment of VGA screen buffer
	mov es, ax ; "Extra Segment" register
	mov al, 0
	
	sub sp, 32 ; space used during run

drawScreen:
	; begin Y loop:
	mov di, 0 ; Onscreen pointer (counts up)
	mov edx, 0 ; Y counter (counts up)
drawrow:
	; begin x loop
	mov ecx, 0 ; X counter (counts up)
drawpixel:
	; choose screen color
	mov [esp+0], ecx ; save counters
	mov [esp+4], edx 
	sub ecx,160   ; put origin in center of screen
	sub edx,100
	imul ecx,3  ; .8 fixed-point scale + geometric scaling
	imul edx,3
	mov [esp+ 8], ecx ; c.real (saved biassed counters)
	mov [esp+12], edx ; c.imag
	
	mov eax,ecx ; z.real
	mov ebx,edx ; z.imag
	mov si,0

mloop:
	; z = z*z + c
	;  z.real = z.real*z.real - z.imag*z.imag + c.real
	;  z.imag = 2*z.real*z.imag + c.imag
	; mag_sqr = z.real*z.real+z.imag*z.imag
	
	; Compute squares
	mov ecx,eax
	imul ecx,ecx
	shr ecx,8 ; ecx==z.real*z.real

	mov edx,ebx
	imul edx,edx
	shr edx,8 ; edx==z.imag*z.imag
	
	; Check magnitude
	mov ebp,ecx
	add ebp,edx ; ebp = mag_sqr
	cmp ebp,1024; ==4.0 in .8 fixed-point
	;jg mbig
	jmp mbig
	
	; Compute new z.real
	mov ebp,ecx
	sub ebp,edx
	add ebp,[esp+8] ; +=c.real
	
	; Compute new z.imag
	imul ebx,eax     ; ebx = z.real*z.imag
	imul ebx,2       ; *=2
	add ebx,[esp+12] ; +=c.imag
	
	mov eax,ebp
	
	; Increment iteration counter
	inc si
	cmp si,256
	jl mloop ; stops when si gets too big	
mbig:
	mov eax,ebp ; Output color==magnitude
	;mov ax,si  ; Output color==counter at magnitude trip
	
	mov ecx,[esp+0] ; restore loop counters
	mov edx,[esp+4]
	
	mov [es:di],al  ; Write this byte to the screen
	inc di
	inc cx
	cmp cx,320
	jl drawpixel
	; end x loop
	
	inc dx
	cmp dx,200
	jl drawrow
	; End Y loop
	
	jmp drawScreen
	
	add sp,32

reboot:
	jmp  0xf000:0xfff0 ; Jump to BIOS reset code

; The boot sector's last two bytes must contain this magic flag.
; This tells the BIOS it's OK to boot here; without this, 
;  you'll get the BIOS "Operating system not found" error.
	times 510-($-_start) db 0  ; pad to offset 510 
	dw 0xAA55 ; Magic value needed by BIOS
END
